@@ -7,10 +7,18 @@ Decode/process a message (payload into JSON).
 """
 
 import logging
+from datetime import timedelta as td
 
 from .const import DONT_CREATE_ENTITIES, DONT_UPDATE_ENTITIES, __dev_mode__
-from .devices import Device
-from .protocol import RAMSES_CODES, CorruptStateError, Message
+from .devices import Device  # , HgiGateway
+from .protocol import (
+    RAMSES_CODES,
+    RAMSES_DEVICES,
+    CorruptStateError,
+    InvalidPacketError,
+    Message,
+)
+from .protocol.message import HVAC_ONLY_CODES
 
 from .protocol import I_, RP, RQ, W_  # noqa: F401, isort: skip
 from .protocol import (  # noqa: F401, isort: skip
@@ -26,6 +34,7 @@ from .protocol import (  # noqa: F401, isort: skip
     _000E,
     _0016,
     _0100,
+    _0150,
     _01D0,
     _01E9,
     _0404,
@@ -34,9 +43,13 @@ from .protocol import (  # noqa: F401, isort: skip
     _0B04,
     _1030,
     _1060,
+    _1081,
     _1090,
+    _1098,
     _10A0,
+    _10B0,
     _10E0,
+    _10E1,
     _1100,
     _1260,
     _1280,
@@ -46,9 +59,12 @@ from .protocol import (  # noqa: F401, isort: skip
     _12B0,
     _12C0,
     _12C8,
+    _12F0,
+    _1300,
     _1F09,
     _1F41,
     _1FC9,
+    _1FD0,
     _1FD4,
     _2249,
     _22C9,
@@ -58,6 +74,11 @@ from .protocol import (  # noqa: F401, isort: skip
     _22F3,
     _2309,
     _2349,
+    _2389,
+    _2400,
+    _2401,
+    _2410,
+    _2420,
     _2D49,
     _2E04,
     _30C9,
@@ -67,7 +88,11 @@ from .protocol import (  # noqa: F401, isort: skip
     _31D9,
     _31DA,
     _31E0,
+    _3200,
+    _3210,
     _3220,
+    _3221,
+    _3223,
     _3B00,
     _3EF0,
     _3EF1,
@@ -81,7 +106,7 @@ CODE_NAMES = {k: v["name"] for k, v in RAMSES_CODES.items()}
 MSG_FORMAT_10 = "|| {:10s} | {:10s} | {:2s} | {:16s} | {:^4s} || {}"
 MSG_FORMAT_18 = "|| {:18s} | {:18s} | {:2s} | {:16s} | {:^4s} || {}"
 
-DEV_MODE = __dev_mode__ and False
+DEV_MODE = __dev_mode__ and False  # set True for useful Tracebacks
 
 _LOGGER = logging.getLogger(__name__)
 if DEV_MODE:
@@ -131,24 +156,109 @@ def _create_devices_from_addrs(this: Message) -> None:
         this._gwy._get_device(this.dst.id, ctl_id=this.src.id, msg=this)  # or _set_ctl?
 
 
-def _create_devices_from_payload(this: Message) -> None:
-    """Discover and create any new devices using the message payload (1FC9/000C)."""
-    pass
+def _check_msg_src(msg: Message, klass: str) -> None:
+    """Validate the packet's source device class (type) against its verb/code pair.
 
-
-def process_msg(msg: Message) -> None:
-    """Process the valid packet by decoding its payload.
-
-    All methods require a valid message (payload), except create_devices, which requires
-    a valid message only for 000C.
+    Raise InvalidPacketError if the meta data is invalid, otherwise simply return.
     """
 
-    # def hack_pkts(this, prev) -> None:  # TODO: needs work, e.g. merge 000A fragments
-    # # TODO: ?move to ctl._handle_msg() and/or system._handle_msg()?
-    # if re.search("I.* 01.* 000A ", str(this._pkt)):  # HACK: and dtm < 3 secs
-    #     # TODO: an edge case here: >2 000A packets in a row
-    #     if prev is not None and re.search("I.* 01.* 000A ", str(prev._pkt)):
-    #         this._payload = prev.payload + this.payload  # merge frags, and process
+    if klass not in RAMSES_DEVICES:  # DEX_done, TODO: fingerprint dev class
+        if msg.code not in HVAC_ONLY_CODES:
+            raise InvalidPacketError(f"Unknown src type: {msg.src}")
+        _LOGGER.warning(f"{msg._pkt} < Unknown src type: {msg.src}, is it HVAC?")
+        return
+
+    #
+    #
+
+    #
+    #
+
+    if msg.code not in RAMSES_DEVICES[klass]:  # DEX_done
+        if klass != "HGI":  # DEX_done
+            raise InvalidPacketError(f"Invalid code for {msg.src} to Tx: {msg.code}")
+        if msg.verb in (RQ, W_):
+            return
+        _LOGGER.warning(f"{msg._pkt} < Invalid code for {msg.src} to Tx: {msg.code}")
+        return
+
+    #
+    #
+    #
+    #
+
+    #
+    # (code := RAMSES_DEVICES[klass][msg.code]) and msg.verb not in code:
+    if msg.verb not in RAMSES_DEVICES[klass][msg.code]:  # DEX_done
+        raise InvalidPacketError(
+            f"Invalid verb/code for {msg.src} to Tx: {msg.verb}/{msg.code}"
+        )
+
+
+def _check_msg_dst(msg: Message, klass: str) -> None:
+    """Validate the packet's destination device class (type) against its verb/code pair.
+
+    Raise InvalidPacketError if the meta data is invalid, otherwise simply return.
+    """
+
+    if klass not in RAMSES_DEVICES:  # DEX_done, TODO: fingerprint dev class
+        if msg.code not in HVAC_ONLY_CODES:
+            raise InvalidPacketError(f"Unknown dst type: {msg.dst}")
+        _LOGGER.warning(f"{msg._pkt} < Unknown dst type: {msg.dst}, is it HVAC?")
+        return
+
+    if msg.verb == I_:  # TODO: not common, unless src=dst
+        return  # receiving an I isn't currently in the schema & cant yet be tested
+
+    if f"{klass}/{msg.verb}/{msg.code}" in (f"CTL/{RQ}/{_3EF1}",):  # DEX_done
+        return  # HACK: an exception-to-the-rule that need sorting
+
+    if msg.code not in RAMSES_DEVICES[klass]:  # NOTE: not OK for Rx, DEX_done
+        if klass != "HGI":  # NOTE: not yet needed because of 1st if, DEX_done
+            raise InvalidPacketError(f"Invalid code for {msg.dst} to Rx: {msg.code}")
+        if msg.verb == RP:
+            return
+        _LOGGER.warning(f"{msg._pkt} < Invalid code for {msg.dst} to Tx: {msg.code}")
+        return
+
+    if f"{msg.verb}/{msg.code}" in (f"{W_}/{_0001}",):
+        return  # HACK: an exception-to-the-rule that need sorting
+    if f"{klass}/{msg.verb}/{msg.code}" in (f"BDR/{RQ}/{_3EF0}",):  # DEX_done
+        return  # HACK: an exception-to-the-rule that need sorting
+
+    verb = {RQ: RP, RP: RQ, W_: I_}[msg.verb]
+    # (code := RAMSES_DEVICES[klass][msg.code]) and verb not in code:
+    if verb not in RAMSES_DEVICES[klass][msg.code]:  # DEX_done
+        raise InvalidPacketError(
+            f"Invalid verb/code for {msg.dst} to Rx: {msg.verb}/{msg.code}"
+        )
+
+
+def process_msg(msg: Message, prev_msg: Message = None) -> None:
+    """Process the valid packet by decoding its payload."""
+
+    # All methods require a valid message (payload), except create_devices(), which
+    # requires a valid message only for 000C.
+
+    def detect_array(this, prev) -> dict:
+        """Return complete array if this pkt is the latter half of an array."""
+        # This will work, even if the 2nd pkt._is_array == False as 1st == True
+        #  I --- 01:158182 --:------ 01:158182 000A 048 001201F409C4011101F409C40...
+        #  I --- 01:158182 --:------ 01:158182 000A 006 081001F409C4
+        if (
+            not prev
+            or not prev._has_array
+            or this.code not in (_000A, _22C9)
+            or this.code != prev.code
+            or this.verb != prev.verb != I_
+            or this.src != prev.src
+            or this.dtm >= prev.dtm + td(seconds=3)
+        ):
+            return this.payload
+
+        msg._pkt._force_has_array()
+        payload = this.payload if isinstance(this.payload, list) else [this.payload]
+        return prev.payload + payload
 
     # HACK:  if CLI, double-logging with client.py proc_msg() & setLevel(DEBUG)
     if (log_level := _LOGGER.getEffectiveLevel()) < logging.INFO:
@@ -159,6 +269,8 @@ def process_msg(msg: Message) -> None:
     # NOTE: this is used to expose message timeouts (esp. when parsing)
     # [m._expired for d in msg._gwy.devices for m in d._msg_db]
 
+    msg._payload = detect_array(msg, prev_msg)  # HACK - needs rethinking?
+
     if msg._gwy.config.reduce_processing >= DONT_CREATE_ENTITIES:
         return
 
@@ -169,18 +281,30 @@ def process_msg(msg: Message) -> None:
 
     try:  # process the packet payload
         _create_devices_from_addrs(msg)  # from pkt header
-        # _create_devices_from_payload(msg)  # from msg payload (e.g. 000C)
+
+        if isinstance(msg.src, Device):
+            _check_msg_src(msg, msg.src._klass)  # ? InvalidPacketError
+        # elif DEV_MODE and not isinstance(msg.src, HgiGateway):
+        #     print(msg)
+        #     print(type(msg.src), msg.src)
+        #     print(type(msg.dst), msg.dst)
+
+        if isinstance(msg.dst, Device):
+            if msg.dst is not msg.src:
+                # Device class doesn't usu. include HgiGateway
+                _check_msg_dst(msg, msg.dst._klass)  # ? InvalidPacketError
+        # elif DEV_MODE and msg.dst.type not in ("18", "63", "--"):
+        #     print(msg)
+        #     print(type(msg.src), msg.src)
+        #     print(type(msg.dst), msg.dst)
 
         if msg._gwy.config.reduce_processing >= DONT_UPDATE_ENTITIES:
-            msg._gwy._prev_msg = msg
             return
 
-        # _update_entities(msg, msg._gwy._prev_msg)  # update the state database
-        if isinstance(msg.src, Device):
+        if isinstance(msg.src, Device):  # , HgiGateway)):  # could use DeviceBase
             msg.src._handle_msg(msg)
 
         if msg.code not in (_0008, _0009, _3B00, _3EF1):  # special case: are fakeable
-            msg._gwy._prev_msg = msg
             return
 
         #  I --- 01:054173 --:------ 01:054173 0008 002 03AA
@@ -209,8 +333,6 @@ def process_msg(msg: Message) -> None:
         )
         return  # NOTE: use raise only when debugging
 
-    except CorruptStateError as exc:  # TODO: add CorruptEvohomeError
+    except (CorruptStateError, InvalidPacketError) as exc:  # TODO: CorruptEvohomeError
         (_LOGGER.exception if DEV_MODE else _LOGGER.error)("%s < %s", msg._pkt, exc)
         return  # TODO: bad pkt, or Schema
-
-    msg._gwy._prev_msg = msg
