@@ -127,9 +127,11 @@ DEVICE_DICT = vol.Schema(
     {
         vol.Optional(DEVICE_ID): vol.Any(
             {
-                vol.Optional(SZ_ALIAS): vol.Any(None, str),
-                vol.Optional(SZ_CLASS): vol.Any(None, *vars(DEV_KLASS).keys()),
-                vol.Optional(SZ_FAKED): vol.Any(None, bool, list),
+                vol.Optional(SZ_ALIAS, default=None): vol.Any(None, str),
+                vol.Optional(SZ_CLASS, default=None): vol.Any(
+                    None, *vars(DEV_KLASS).keys()
+                ),
+                vol.Optional(SZ_FAKED, default=None): vol.Any(None, bool),
             },
         )
     },
@@ -137,15 +139,13 @@ DEVICE_DICT = vol.Schema(
 )
 
 # 2/3: Schemas for Heating systems
-SZ_SYS_PROFILE = "_profile"
-SYSTEM_PROFILES = (SystemType.EVOHOME, SystemType.HOMETRONICS, SystemType.SUNDIAL)
+SZ_SYS_KLASS = "class"
+SYSTEM_KLASS = (SystemType.EVOHOME, SystemType.HOMETRONICS, SystemType.SUNDIAL)
 
 HTG_SCHEMA = vol.Schema(
     {
         vol.Required(SZ_HTG_CONTROL, default=None): vol.Any(None, DEV_REGEX_HTG),
-        vol.Optional(SZ_SYS_PROFILE, default=SystemType.EVOHOME): vol.Any(
-            *SYSTEM_PROFILES
-        ),
+        vol.Optional(SZ_SYS_KLASS, default=SystemType.EVOHOME): vol.Any(*SYSTEM_KLASS),
     },
     # extra=vol.ALLOW_EXTRA,  # TODO: remove me
 )
@@ -249,12 +249,9 @@ def load_config(
     update_config(config, known_list, block_list)
     config = SimpleNamespace(**config)
 
-    # # TODO: remove
     # config.use_regex.update(
-    #     {
-    #         "( 03:.* 03:.* (1060|2389|30C9) 003) ..": "\\1 00",
-    #     }
-    # )
+    #     {"( 03:.* 03:.* (1060|2389|30C9) 003) ..": "\\1 00"}
+    # )  # TODO: remove - for testing only
 
     return (config, schema, known_list, block_list)
 
@@ -297,7 +294,9 @@ def update_config(config, known_list, block_list) -> dict:
         )
 
 
-def _get_device(gwy, dev_id, ctl_id=None, **kwargs) -> Optional[Any]:  # -> Device:
+def _get_device(
+    gwy, dev_id, ctl_id=None, disable_warning=None, **kwargs
+) -> Optional[Any]:  # -> Device:
     """Get (optionally create) a device only if not filtered out."""
 
     if "dev_addr" in kwargs or "ctl_addr" in kwargs:
@@ -311,7 +310,9 @@ def _get_device(gwy, dev_id, ctl_id=None, **kwargs) -> Optional[Any]:  # -> Devi
         err_msg = None
 
     if err_msg:
-        _LOGGER.error(f"{err_msg}: check the lists and the {SCHEMA} (device ignored)")
+        (_LOGGER.info if disable_warning else _LOGGER.error)(
+            f"{err_msg}: check the lists and the {SCHEMA} (device not created)"
+        )
         return
 
     return gwy._get_device(dev_id, ctl_id=ctl_id, **kwargs)
@@ -331,13 +332,16 @@ def load_schema(gwy, **kwargs) -> dict:
     if kwargs.get(SZ_MAIN_CONTROLLER):
         gwy.evo = gwy.system_by_id.get(kwargs[SZ_MAIN_CONTROLLER])
 
-    [_get_device(gwy, device_id) for device_id in kwargs.pop(SZ_ORPHANS, [])]
+    [
+        _get_device(gwy, device_id, disable_warning=True)
+        for device_id in kwargs.pop(SZ_ORPHANS, [])
+    ]
 
 
 def load_system(gwy, ctl_id, schema) -> Tuple[dict, dict]:
     schema = SYSTEM_SCHEMA(schema)
 
-    if (ctl := _get_device(gwy, ctl_id, ctl_id=ctl_id, profile=None)) is None:
+    if (ctl := _get_device(gwy, ctl_id, ctl_id=ctl_id)) is None:
         return
 
     if dev_id := schema[SZ_HTG_SYSTEM].get(SZ_HTG_CONTROL):
@@ -372,7 +376,7 @@ def load_system(gwy, ctl_id, schema) -> Tuple[dict, dict]:
         _get_device(gwy, dev_id, ctl_id=ctl.id)  # , **_schema)
 
     for dev_id in schema.get(SZ_ORPHANS, []):
-        _get_device(gwy, dev_id, ctl_id=ctl.id)
+        _get_device(gwy, dev_id, ctl_id=ctl.id, disable_warning=True)
 
     if False and DEV_MODE:
         import json
