@@ -9,7 +9,7 @@ from datetime import datetime as dt
 from datetime import timedelta as td
 from threading import Lock
 from types import SimpleNamespace
-from typing import List, Optional
+from typing import List, Optional, Tuple
 
 from .const import (
     _000C_DEVICE,
@@ -21,7 +21,6 @@ from .const import (
     ATTR_SYSTEM_MODE,
     SYSTEM_MODE,
     Discover,
-    SystemType,
     __dev_mode__,
 )
 from .devices import (
@@ -33,13 +32,7 @@ from .devices import (
     UfhController,
 )
 from .entities import Entity, class_by_attr, discover_decorator
-from .protocol import (
-    Command,
-    CorruptStateError,
-    ExpiredCallbackError,
-    FaultLog,
-    Priority,
-)
+from .protocol import Command, CorruptStateError, ExpiredCallbackError, Priority
 from .protocol.transport import PacketProtocolPort
 from .schema import (
     SZ_CONTROLLER,
@@ -70,6 +63,7 @@ from .protocol import (  # noqa: F401, isort: skip
     _000E,
     _0016,
     _0100,
+    _0150,
     _01D0,
     _01E9,
     _0404,
@@ -78,9 +72,13 @@ from .protocol import (  # noqa: F401, isort: skip
     _0B04,
     _1030,
     _1060,
+    _1081,
     _1090,
+    _1098,
     _10A0,
+    _10B0,
     _10E0,
+    _10E1,
     _1100,
     _1260,
     _1280,
@@ -90,9 +88,12 @@ from .protocol import (  # noqa: F401, isort: skip
     _12B0,
     _12C0,
     _12C8,
+    _12F0,
+    _1300,
     _1F09,
     _1F41,
     _1FC9,
+    _1FD0,
     _1FD4,
     _2249,
     _22C9,
@@ -102,6 +103,11 @@ from .protocol import (  # noqa: F401, isort: skip
     _22F3,
     _2309,
     _2349,
+    _2389,
+    _2400,
+    _2401,
+    _2410,
+    _2420,
     _2D49,
     _2E04,
     _30C9,
@@ -111,7 +117,11 @@ from .protocol import (  # noqa: F401, isort: skip
     _31D9,
     _31DA,
     _31E0,
+    _3200,
+    _3210,
     _3220,
+    _3221,
+    _3223,
     _3B00,
     _3EF0,
     _3EF1,
@@ -125,9 +135,10 @@ if DEV_MODE:
     _LOGGER.setLevel(logging.DEBUG)
 
 
-SYSTEM_PROFILE = SimpleNamespace(
-    EVO="evohome",  # Stored HW (not a zone)
-    PRG="programmer",  # Electric
+SYS_KLASS = SimpleNamespace(
+    SYS="system",  # Generic (promotable?) system
+    EVO="evohome",
+    PRG="programmer",
 )
 
 
@@ -172,7 +183,7 @@ class SystemBase(Entity):  # 3B00 (multi-relay)
             self._discover, discover_flag=Discover.STATUS, delay=2, period=60
         )
         self._gwy._add_task(
-            self._discover, discover_flag=Discover.FAULTS, delay=120, period=60
+            self._discover, discover_flag=Discover.FAULTS, delay=60, period=60
         )
         self._gwy._add_task(
             self._discover, discover_flag=Discover.SCHEDS, delay=300, period=60
@@ -203,14 +214,14 @@ class SystemBase(Entity):  # 3B00 (multi-relay)
         assert msg.src is self._ctl, f"msg inappropriately routed to {self}"
         super()._handle_msg(msg)
 
-        if msg.code == _0008 and msg.verb in (I_, RP):
-            if "domain_id" in msg.payload:
-                self._relay_demands[msg.payload["domain_id"]] = msg
-                if msg.payload["domain_id"] == "F9":
+        if msg.code == _0008:
+            if (domain_id := msg.payload.get("domain_id")) and msg.verb in (I_, RP):
+                self._relay_demands[domain_id] = msg
+                if domain_id == "F9":
                     device = self.dhw.heating_valve if self.dhw else None
-                elif msg.payload["domain_id"] == "FA":
+                elif domain_id == "FA":
                     device = self.dhw.hotwater_valve if self.dhw else None
-                elif msg.payload["domain_id"] == "FC":
+                elif domain_id == "FC":
                     device = self.heating_control
                 else:
                     device = None
@@ -220,8 +231,13 @@ class SystemBase(Entity):  # 3B00 (multi-relay)
                     for code in (_0008, _3EF1):
                         device._make_cmd(code, qos)
 
-        if msg.code == _3150 and msg.verb in (I_, RP):
-            if "domain_id" in msg.payload and msg.payload["domain_id"] == "FC":
+        elif msg.code == _000C:
+            if msg.payload["device_class"] == SZ_HTG_CONTROL and msg.payload["devices"]:
+                self._set_htg_control(self._ctl.device_by_id[msg.payload["devices"][0]])
+            return
+
+        elif msg.code == _3150:
+            if msg.payload.get("domain_id") == "FC" and msg.verb in (I_, RP):
                 self._heat_demand = msg.payload
 
         if self._gwy.config.enable_eavesdrop and not self.heating_control:
@@ -450,8 +466,8 @@ class MultiZone:  # 0005 (+/- 000C?)
                 handle_msg_by_zone_idx(zone_idx, msg)
             # TODO: elif msg.payload.get("domain_id") == "FA":  # DHW
 
-        elif isinstance(msg.payload, list):
-            if msg.payload[0].get("zone_idx"):
+        elif isinstance(msg.payload, list) and len(msg.payload):
+            if isinstance(msg.payload[0], dict):  # e.g. 1FC9 is a list of lists:
                 [handle_msg_by_zone_idx(z.get("zone_idx"), msg) for z in msg.payload]
             # TODO: elif msg.payload.get("domain_id") == "FA":  # DHW
 
@@ -704,9 +720,14 @@ class Logbook:  # 0418
     def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
 
-        self._prev_0418 = None
-        self._this_0418 = None
-        self._faultlog = FaultLog(self._ctl)
+        self._prev_event = None
+        self._this_event = None
+
+        self._prev_fault = None
+        self._this_fault = None
+
+        self._faultlog = None  # FaultLog(self._ctl)
+        self._faultlog_outdated = None  # should be True
 
     def _discover(self, discover_flag=Discover.ALL) -> None:
         super()._discover(discover_flag=discover_flag)
@@ -718,12 +739,30 @@ class Logbook:  # 0418
     def _handle_msg(self, msg) -> None:  # NOTE: active
         super()._handle_msg(msg)
 
-        if msg.code == _0418 and msg.payload["log_idx"] == "00":
-            self._this_0418 = msg
-            if self._faultlog_outdated:
-                self._prev_0418 = msg
-                if not self._gwy.config.disable_sending:
-                    self._loop.create_task(self.get_faultlog(force_refresh=True))
+        if msg.code != _0418:
+            return
+
+        if msg.payload["log_idx"] == "00":
+            if not self._this_event or (
+                msg.payload["log_entry"] != self._this_event.payload["log_entry"]
+            ):
+                self._this_event, self._prev_event = msg, self._this_event
+            # TODO: self._faultlog_outdated = msg.verb == I_ or self._prev_event and (
+            #     msg.payload["log_entry"] != self._prev_event.payload["log_entry"]
+            # )
+
+        if msg.payload["log_entry"] and msg.payload["log_entry"][1] == "fault":
+            if not self._this_fault or (
+                msg.payload["log_entry"] != self._this_fault.payload["log_entry"]
+            ):
+                self._this_fault, self._prev_fault = msg, self._this_fault
+
+        # if msg.payload["log_entry"][1] == "restore" and not self._this_fault:
+        #     self._send_cmd(Command.get_system_log_entry(self._ctl.id, 1))
+
+        # TODO: if self._faultlog_outdated:
+        #     if not self._gwy.config.disable_sending:
+        #         self._loop.create_task(self.get_faultlog(force_refresh=True))
 
     async def get_faultlog(self, start=None, limit=None, force_refresh=None) -> dict:
         if self._gwy.config.disable_sending:
@@ -736,23 +775,39 @@ class Logbook:  # 0418
         except (ExpiredCallbackError, RuntimeError):
             return
 
-    @property
-    def _faultlog_outdated(self) -> bool:
-        return (
-            not self._prev_0418
-            or not self._this_0418
-            or (self._prev_0418._pkt.payload != self._this_0418._pkt.payload)
-        )
+    # @property
+    # def faultlog_outdated(self) -> bool:
+    #     return self._this_event.verb == I_ or self._prev_event and (
+    #         self._this_event.payload != self._prev_event.payload
+    #     )
+
+    # @property
+    # def faultlog(self) -> dict:
+    #     return self._faultlog.faultlog
 
     @property
-    def faultlog(self) -> dict:
-        return self._faultlog.faultlog
+    def active_fault(self) -> Optional[Tuple]:
+        """Return the most recently logged event, but only if it is a fault."""
+        if self.latest_fault == self.latest_event:
+            return self.latest_fault
+
+    @property
+    def latest_event(self) -> Optional[Tuple]:
+        """Return the most recently logged event (fault or restore), if any."""
+        return self._this_event and self._this_event.payload["log_entry"]
+
+    @property
+    def latest_fault(self) -> Optional[Tuple]:
+        """Return the most recently logged fault, if any."""
+        return self._this_fault and self._this_fault.payload["log_entry"]
 
     @property
     def status(self) -> dict:
         return {
             **super().status,
-            "faultlog": self.faultlog,
+            "latest_event": self.latest_event,
+            "active_fault": self.active_fault,
+            # "faultlog": self.faultlog,
         }
 
 
@@ -1006,7 +1061,7 @@ class UfHeating:
 class System(StoredHw, Datetime, Logbook, SystemBase):
     """The Controller class."""
 
-    _PROFILE = SYSTEM_PROFILE.PRG
+    _SYS_KLASS = SYS_KLASS.PRG
 
     def __init__(self, gwy, ctl, **kwargs) -> None:
         super().__init__(gwy, ctl, **kwargs)
@@ -1070,43 +1125,15 @@ class Evohome(ScheduleSync, Language, SysMode, MultiZone, UfHeating, System):
 
     # older evohome don't have zone_type=ELE
 
-    _PROFILE = SYSTEM_PROFILE.EVO
+    _SYS_KLASS = SYS_KLASS.EVO
 
     def __repr__(self) -> str:
         return f"{self._ctl.id} (evohome)"
 
-    # def _discover(self, discover_flag=Discover.ALL) -> None:
-    #     super()._discover(discover_flag=discover_flag)
-
-    #     if discover_flag & Discover.STATUS:
-    #         self._make_cmd(_1F09)
-
-    def _handle_msg(self, msg) -> None:
-        super()._handle_msg(msg)
-
-        # def xxx(zone_dict):
-        #     zone = self.zone_by_idx[zone_dict.pop("zone_idx")]
-        #     if msg.code == _000A:
-        #         zone._zone_config = zone_dict
-        #     elif msg.code == _2309:
-        #         zone._temp = zone_dict
-        #     elif msg.code == _30C9:
-        #         zone._temp = zone_dict
-
-        # if msg.code in (_000A, _2309, _30C9):
-        #     if isinstance(msg.payload, list):
-        #         super()._handle_msg(msg)
-        #         [xxx(z) for z in msg.payload]
-        #     else:
-        #         xxx(msg.payload)
-
-        if msg.code in (_000A, _2309, _30C9) and isinstance(msg.payload, list):
-            pass
-
 
 class Chronotherm(Evohome):
 
-    _PROFILE = SYSTEM_PROFILE.EVO
+    _SYS_KLASS = SYS_KLASS.SYS
 
     def __repr__(self) -> str:
         return f"{self._ctl.id} (chronotherm)"
@@ -1121,7 +1148,7 @@ class Hometronics(System):
 
     # Hometronic does not react to W/2349 but rather requies W/2309
 
-    _PROFILE = SYSTEM_PROFILE.EVO
+    _SYS_KLASS = SYS_KLASS.SYS
 
     RQ_SUPPORTED = (_0004, _000C, _2E04, _313F)  # TODO: WIP
     RQ_UNSUPPORTED = ("xxxx",)  # 10E0?
@@ -1142,7 +1169,7 @@ class Hometronics(System):
 
 class Programmer(Evohome):
 
-    _PROFILE = SYSTEM_PROFILE.PRG
+    _SYS_KLASS = SYS_KLASS.PRG
 
     def __repr__(self) -> str:
         return f"{self._ctl.id} (programmer)"
@@ -1150,44 +1177,26 @@ class Programmer(Evohome):
 
 class Sundial(Evohome):
 
-    _PROFILE = SYSTEM_PROFILE.PRG
+    _SYS_KLASS = SYS_KLASS.SYS
 
     def __repr__(self) -> str:
         return f"{self._ctl.id} (sundial)"
 
 
-_SYS_CLASS = {
-    SystemType.CHRONOTHERM: Chronotherm,
-    SystemType.EVOHOME: Evohome,
-    SystemType.HOMETRONICS: Hometronics,
-    SystemType.PROGRAMMER: Programmer,
-    SystemType.SUNDIAL: Sundial,
-    SystemType.GENERIC: System,
-}
-_SYS_CLASS_BY_TYPE = {
-    "01": Evohome,
-    "23": Programmer,
-    "12": Sundial,
-}
+_SYS_BY_KLASS = class_by_attr(__name__, "_SYS_KLASS")  # e.g. "evohome": Evohome
 
 
-_SYS_CLASS_BY_PROFILE = class_by_attr(__name__, "_PROFILE")  # e.g. "evohome": Evohome
-
-
-def create_system(gwy, ctl, profile=None, **kwargs) -> System:
+def create_system(gwy, ctl, klass=None, **kwargs) -> System:
     """Create a system, and optionally perform discovery & start polling."""
 
-    #
+    if klass is None:
+        klass = SYS_KLASS.PRG if isinstance(ctl, Programmer) else SYS_KLASS.EVO
 
-    if profile is None:  # TODO: messy
-        profile = (
-            SYSTEM_PROFILE.PRG if isinstance(ctl, Programmer) else SYSTEM_PROFILE.EVO
-        )
-        #
+    system = _SYS_BY_KLASS.get(klass, System)(gwy, ctl, **kwargs)
 
-    system = _SYS_CLASS.get(profile, System)(gwy, ctl, **kwargs)
-
-    if isinstance(gwy.pkt_protocol, PacketProtocolPort):
+    if not gwy.config.disable_discovery and isinstance(
+        gwy.pkt_protocol, PacketProtocolPort
+    ):
         system._start_discovery()
 
     return system
